using namespace std;

#include <iostream>
#include <unordered_set>
#include <unordered_map>
#include "testlib.h"

typedef long long ll;

int max_n = 100000;
int max_m = 200000;
int max_d = 1000000;
int max_a = 1000000;
ll max_x = 1000000000LL;

typedef tuple<int, ll, ll> room;

void check_connectivity(const vector<vector<int>> &tree, int v, vector<bool> &used) {
    used[v] = true;
    for (int u : tree[v]) {
        check_connectivity(tree, u, used);
    }
}

int main(int argc, char *argv[]) {
    registerValidation(argc, argv);

    int n = inf.readInt(2, max_n, "n");
    inf.readSpace();
    int m = inf.readInt(1, max_m, "m");
    inf.readEoln();
    vector <room> rooms(n + 1);
    vector <vector<int>> rooms_child(n + 1);
    pair<ll, int> real_max_d(0, 0);
    for (int i = 0; i < n; ++i) {
        get<0>(rooms[i + 1]) = inf.readInt(0, n, "b_i");
        inf.readSpace();
        get<1>(rooms[i + 1]) = inf.readInt(0, max_d, "d_i");
        inf.readSpace();
        get<2>(rooms[i + 1]) = inf.readInt(0, max_a, "a_i");
        inf.readEoln();
        real_max_d = max(real_max_d, make_pair(get<1>(rooms[i + 1]), i + 1));
        rooms_child[get<0>(rooms[i + 1])].push_back(i + 1);
    }
    for (int i = 1; i <= n; ++i) {
        if (get<1>(rooms[i]) == real_max_d.first) {
            ensuref(i == real_max_d.second, "there are two lowest rooms: %d, %d", i, real_max_d.second);
        }
    }
    for (int i = 1; i <= n; ++i) {
        if (get<0>(rooms[i]) == 0) { continue; }
        ensuref(get<1>(rooms[get<0>(rooms[i])]) > get<1>(rooms[i]), "incorrect tube: %d->%d", i, get<0>(rooms[i]));
    }
    vector<bool> visited(n + 1, false);
    check_connectivity(rooms_child, real_max_d.second, visited);
    for (int i = 1; i <= n; ++i) {
        ensuref(visited[i], "no visit %d, because start from %d", i, real_max_d.second);

        if (rooms_child[i].empty()) {
            ensuref(get<1>(rooms[i]) == 0, "room %d with out children, but have depth %lld", i, get<1>(rooms[i]));
        }
    }

    for (int i = 0; i < m; ++i) {
        int r = inf.readInt(1, n, "r_j");
        inf.readSpace();
        int x = inf.readInt(1, max_x, "x_j");
        inf.readEoln();
    }

    if (validator.group() == "1") {
        ensuref(n <= 100 && m <= 100, "g1: n, m must be <= 100");
        for (int i = 1; i <= n; ++i) {
            ensuref(get<1>(rooms[i]) <= 100, "g1: d_%d must be <= 100", i);
            ensuref(get<2>(rooms[i]) <= 100, "g1: a_%d must be <= 100", i);
        }
    }
    if (validator.group() == "2") {
        set <ll> bs;
        for (int i = 1; i <= n; ++i) {
            bs.insert(get<1>(rooms[i]));
        }
        ensuref(bs.size() == n, "g2: b_i must be all not equal");
    }
    if (validator.group() == "3") {
        ll tube_len = get<0>(rooms[1]) > 0 ?
                      (get<1>(rooms[get<0>(rooms[1])]) - get<1>(rooms[1])) :
                      (get<1>(rooms[get<0>(rooms[2])]) - get<1>(rooms[2]));
        for (int i = 1; i <= n; ++i) {
            if (get<0>(rooms[i]) > 0) {
                ensuref(tube_len == (get<1>(rooms[get<0>(rooms[i])]) - get<1>(rooms[i])),
                        "g3: tube length must be equal in %d", i);
            }
            if (get<1>(rooms[i]) > 0) {
                ensuref(rooms_child[i].size() == 2,
                        "g3: room %d length must have two children, but has %d", i, (int)rooms_child[i].size());
            }
        }
    }
    if (validator.group() == "4") {
        ensuref(n <= 100 && m <= 100, "g4: n, m must be <= 100");
    }
    if (validator.group() == "5") {
        ensuref(n <= 2000 && m <= 2000, "g4: n, m must be <= 100");
    }

    inf.readEof();
    return 0;
}
