#include "testlib.h"

using namespace std;


const int infinity = 1e9 + 1;


struct MinStack {
    struct Element {
        int ind, val, min1, min2;

        Element(int i, int v, Element pr) {
            ind = i;
            val = v;
            if (v <= pr.min1) {
                min2 = pr.min1;
                min1 = v;
            } else if (v <= pr.min2) {
                min1 = pr.min1;
                min2 = v;
            } else {
                min1 = pr.min1;
                min2 = pr.min2;
            }
        }

        Element(int i, int v) {
            ind = i;
            val = v;
            min1 = v;
            min2 = infinity;
        }
    };

    vector<Element> st;

    bool isEmpty() {
        return st.empty();
    }

    void push(int ind, int priority) {
        if (st.empty()) {
            st.emplace_back(Element(ind, priority));
        } else {
            st.emplace_back(Element(ind, priority, st.back()));
        }
    }

    pair<int, int> pop() {
        if (st.empty()) {
            return {infinity, infinity};
        }
        pair<int, int> res = {st.back().ind, st.back().val};
        st.pop_back();
        return res;
    }

    int min1() {
        if (st.empty()) {
            return infinity;
        }
        return st.back().min1;
    }

    int min2() {
        if (st.empty()) {
            return infinity;
        }
        return st.back().min2;
    }

    pair<int, int> getPair() {
        return {min1(), min2()};
    }
};


pair<int, int> mergeMin(pair<int, int> m1, pair<int, int> m2) {
    if (m1.first <= m2.first) {
        return {m1.first, min(m1.second, m2.first)};
    } else {
        return {m2.first, min(m1.first, m2.second)};
    }
}


struct MinQueue {
    MinStack s1, s2;

    void push(int ind, int prior) {
        s2.push(ind, prior);
    }

    pair<int, int> pop() {
        if (s1.isEmpty()) {
            while (!s2.isEmpty()) {
                auto val = s2.pop();
                s1.push(val.first, val.second);
            }
        }
        return s1.pop();
    }

    bool isEmpty() {
        return s1.isEmpty() and s2.isEmpty();
    }

    int min1() {
        return min(s1.min1(), s2.min1());
    }

    int min2() {
        return mergeMin(s1.getPair(), s2.getPair()).second;
    }

    pair<int, int> getPair() {
        return mergeMin(s1.getPair(), s2.getPair());
    }
};


MinQueue checked, unchecked;


void scream() {
    auto twoMins = checked.getPair();
    while (!unchecked.isEmpty()) {
        auto cnt = unchecked.pop();
        if (cnt.second <= twoMins.second) {
            checked.push(cnt.first, cnt.second);
        }
        twoMins = mergeMin(twoMins, {cnt.second, infinity});
    }
}


bool isEmpty() {
    return checked.isEmpty() and unchecked.isEmpty();
}


void push(int ind, int p, int X) {
    auto twoMins = mergeMin(checked.getPair(), unchecked.getPair());
    if (twoMins.second < p && (long long)(p - twoMins.first) * (p - twoMins.second) > X) {
        scream();
    } else {
        unchecked.push(ind, p);
    }
}


void pop() {
    if (!checked.isEmpty()) {
        checked.pop();
    } else {
        unchecked.pop();
    }
}

int main(int argc, char *argv[]) {
    registerValidation(argc, argv);
    int n = inf.readInt();
    inf.readSpace();
    int X = inf.readInt(1, 1'000'000'000);
    set<int> used;
    inf.readEoln();
    for (int i = 0; i < n; i++) {
        char c = inf.readChar();
        if (c == '-') {
            ensuref(!isEmpty(), "Operation %i is bad: cannot pop from empty queue", i + 1);
            pop();
            inf.readEoln();
        } else {
            inf.readSpace();
            int ind = inf.readInt();
            ensuref(used.count(ind) == 0, "Index collision: %i met twice", ind);
            inf.readSpace();
            int p = inf.readInt();
            push(ind, p, X);
            inf.readEoln();
        }
    }
    inf.readEof();
}